package javax.swing.plaf.synth;

import javax.swing.Box;
import javax.swing.JSeparator;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Container;
import java.beans.PropertyChangeEvent;
import sun.swing.plaf.synth.SynthIcon;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.LayoutManager;
import javax.swing.JToolBar;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Rectangle;
import javax.swing.Icon;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicToolBarUI;

public class SynthToolBarUI extends BasicToolBarUI implements PropertyChangeListener, SynthUI
{
    private Icon handleIcon;
    private Rectangle contentRect;
    private SynthStyle style;
    private SynthStyle contentStyle;
    private SynthStyle dragWindowStyle;
    
    public SynthToolBarUI() {
        this.handleIcon = null;
        this.contentRect = new Rectangle();
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthToolBarUI();
    }
    
    @Override
    protected void installDefaults() {
        this.toolBar.setLayout(this.createLayout());
        this.updateStyle(this.toolBar);
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.toolBar.addPropertyChangeListener(this);
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.toolBar.removePropertyChangeListener(this);
    }
    
    private void updateStyle(JToolBar toolBar) {
        SynthContext context = this.getContext(toolBar, Region.TOOL_BAR_CONTENT, null, 1);
        this.contentStyle = SynthLookAndFeel.updateStyle(context, this);
        context.dispose();
        SynthContext context2 = this.getContext(toolBar, Region.TOOL_BAR_DRAG_WINDOW, null, 1);
        this.dragWindowStyle = SynthLookAndFeel.updateStyle(context2, this);
        context2.dispose();
        SynthContext context3 = this.getContext(toolBar, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context3, this);
        if (style != this.style) {
            this.handleIcon = this.style.getIcon(context3, "ToolBar.handleIcon");
            if (style != null) {
                this.uninstallKeyboardActions();
                this.installKeyboardActions();
            }
        }
        context3.dispose();
    }
    
    @Override
    protected void uninstallDefaults() {
        SynthContext context = this.getContext(this.toolBar, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
        this.handleIcon = null;
        SynthContext context2 = this.getContext(this.toolBar, Region.TOOL_BAR_CONTENT, this.contentStyle, 1);
        this.contentStyle.uninstallDefaults(context2);
        context2.dispose();
        this.contentStyle = null;
        SynthContext context3 = this.getContext(this.toolBar, Region.TOOL_BAR_DRAG_WINDOW, this.dragWindowStyle, 1);
        this.dragWindowStyle.uninstallDefaults(context3);
        context3.dispose();
        this.dragWindowStyle = null;
        this.toolBar.setLayout(null);
    }
    
    @Override
    protected void installComponents() {
    }
    
    @Override
    protected void uninstallComponents() {
    }
    
    protected LayoutManager createLayout() {
        return new SynthToolBarUI.SynthToolBarLayoutManager();
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, SynthLookAndFeel.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private SynthContext getContext(JComponent component, Region region, SynthStyle synthStyle) {
        return SynthContext.getContext(component, region, synthStyle, this.getComponentState(component, region));
    }
    
    private SynthContext getContext(JComponent component, Region region, SynthStyle synthStyle, int n) {
        return SynthContext.getContext(component, region, synthStyle, n);
    }
    
    private int getComponentState(JComponent component, Region region) {
        return SynthLookAndFeel.getComponentState(component);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintToolBarBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight(), this.toolBar.getOrientation());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintToolBarBorder(synthContext, graphics, n, n2, n3, n4, this.toolBar.getOrientation());
    }
    
    @Override
    protected void setBorderToNonRollover(Component component) {
    }
    
    @Override
    protected void setBorderToRollover(Component component) {
    }
    
    @Override
    protected void setBorderToNormal(Component component) {
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
        if (this.handleIcon != null && this.toolBar.isFloatable()) {
            SynthIcon.paintIcon(this.handleIcon, synthContext, graphics, this.toolBar.getComponentOrientation().isLeftToRight() ? 0 : (this.toolBar.getWidth() - SynthIcon.getIconWidth(this.handleIcon, synthContext)), 0, SynthIcon.getIconWidth(this.handleIcon, synthContext), SynthIcon.getIconHeight(this.handleIcon, synthContext));
        }
        SynthContext context = this.getContext(this.toolBar, Region.TOOL_BAR_CONTENT, this.contentStyle);
        this.paintContent(context, graphics, this.contentRect);
        context.dispose();
    }
    
    protected void paintContent(SynthContext synthContext, Graphics graphics, Rectangle rectangle) {
        SynthLookAndFeel.updateSubregion(synthContext, graphics, rectangle);
        synthContext.getPainter().paintToolBarContentBackground(synthContext, graphics, rectangle.x, rectangle.y, rectangle.width, rectangle.height, this.toolBar.getOrientation());
        synthContext.getPainter().paintToolBarContentBorder(synthContext, graphics, rectangle.x, rectangle.y, rectangle.width, rectangle.height, this.toolBar.getOrientation());
    }
    
    @Override
    protected void paintDragWindow(Graphics graphics) {
        int width = this.dragWindow.getWidth();
        int height = this.dragWindow.getHeight();
        SynthContext context = this.getContext(this.toolBar, Region.TOOL_BAR_DRAG_WINDOW, this.dragWindowStyle);
        SynthLookAndFeel.updateSubregion(context, graphics, new Rectangle(0, 0, width, height));
        context.getPainter().paintToolBarDragWindowBackground(context, graphics, 0, 0, width, height, this.dragWindow.getOrientation());
        context.getPainter().paintToolBarDragWindowBorder(context, graphics, 0, 0, width, height, this.dragWindow.getOrientation());
        context.dispose();
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JToolBar)propertyChangeEvent.getSource());
        }
    }
    
    class SynthToolBarLayoutManager implements LayoutManager
    {
        @Override
        public void addLayoutComponent(String s, Component component) {
        }
        
        @Override
        public void removeLayoutComponent(Component component) {
        }
        
        @Override
        public Dimension minimumLayoutSize(Container container) {
            JToolBar toolBar = (JToolBar)container;
            Insets insets = toolBar.getInsets();
            Dimension dimension = new Dimension();
            SynthContext context = SynthToolBarUI.this.getContext(toolBar);
            if (toolBar.getOrientation() == 0) {
                dimension.width = (toolBar.isFloatable() ? SynthIcon.getIconWidth(SynthToolBarUI.this.handleIcon, context) : 0);
                for (int j = 0; j < toolBar.getComponentCount(); ++j) {
                    Component component2 = toolBar.getComponent(j);
                    if (component2.isVisible()) {
                        Dimension minimumSize2 = component2.getMinimumSize();
                        dimension.width += minimumSize2.width;
                        dimension.height = Math.max(dimension.height, minimumSize2.height);
                    }
                }
            }
            else {
                dimension.height = (toolBar.isFloatable() ? SynthIcon.getIconHeight(SynthToolBarUI.this.handleIcon, context) : 0);
                for (int i = 0; i < toolBar.getComponentCount(); ++i) {
                    Component component = toolBar.getComponent(i);
                    if (component.isVisible()) {
                        Dimension minimumSize = component.getMinimumSize();
                        dimension.width = Math.max(dimension.width, minimumSize.width);
                        dimension.height += minimumSize.height;
                    }
                }
            }
            dimension.width += insets.left + insets.right;
            dimension.height += insets.top + insets.bottom;
            context.dispose();
            return dimension;
        }
        
        @Override
        public Dimension preferredLayoutSize(Container container) {
            JToolBar toolBar = (JToolBar)container;
            Insets insets = toolBar.getInsets();
            Dimension dimension = new Dimension();
            SynthContext context = SynthToolBarUI.this.getContext(toolBar);
            if (toolBar.getOrientation() == 0) {
                dimension.width = (toolBar.isFloatable() ? SynthIcon.getIconWidth(SynthToolBarUI.this.handleIcon, context) : 0);
                for (int j = 0; j < toolBar.getComponentCount(); ++j) {
                    Component component2 = toolBar.getComponent(j);
                    if (component2.isVisible()) {
                        Dimension preferredSize2 = component2.getPreferredSize();
                        dimension.width += preferredSize2.width;
                        dimension.height = Math.max(dimension.height, preferredSize2.height);
                    }
                }
            }
            else {
                dimension.height = (toolBar.isFloatable() ? SynthIcon.getIconHeight(SynthToolBarUI.this.handleIcon, context) : 0);
                for (int i = 0; i < toolBar.getComponentCount(); ++i) {
                    Component component = toolBar.getComponent(i);
                    if (component.isVisible()) {
                        Dimension preferredSize = component.getPreferredSize();
                        dimension.width = Math.max(dimension.width, preferredSize.width);
                        dimension.height += preferredSize.height;
                    }
                }
            }
            dimension.width += insets.left + insets.right;
            dimension.height += insets.top + insets.bottom;
            context.dispose();
            return dimension;
        }
        
        @Override
        public void layoutContainer(Container container) {
            JToolBar toolBar = (JToolBar)container;
            Insets insets = toolBar.getInsets();
            int leftToRight = toolBar.getComponentOrientation().isLeftToRight();
            SynthContext context = SynthToolBarUI.this.getContext(toolBar);
            int n = 0;
            for (int i = 0; i < toolBar.getComponentCount(); ++i) {
                if (this.isGlue(toolBar.getComponent(i))) {
                    ++n;
                }
            }
            if (toolBar.getOrientation() == 0) {
                int n3 = toolBar.isFloatable() ? SynthIcon.getIconWidth(SynthToolBarUI.this.handleIcon, context) : 0;
                SynthToolBarUI.this.contentRect.x = (leftToRight ? n3 : false);
                SynthToolBarUI.this.contentRect.y = 0;
                SynthToolBarUI.this.contentRect.width = toolBar.getWidth() - n3;
                SynthToolBarUI.this.contentRect.height = toolBar.getHeight();
                int n4 = leftToRight ? (n3 + insets.left) : (toolBar.getWidth() - n3 - insets.right);
                int top = insets.top;
                int height = toolBar.getHeight() - insets.top - insets.bottom;
                int n5 = 0;
                if (n > 0) {
                    n5 = (toolBar.getWidth() - this.minimumLayoutSize(container).width) / n;
                    if (n5 < 0) {
                        n5 = 0;
                    }
                }
                for (int k = 0; k < toolBar.getComponentCount(); ++k) {
                    Component component2 = toolBar.getComponent(k);
                    if (component2.isVisible()) {
                        Dimension preferredSize2 = component2.getPreferredSize();
                        if (preferredSize2.height >= height || component2 instanceof JSeparator) {
                        }
                        else {
                            top = top + height / 2 - preferredSize2.height / 2;
                            height = preferredSize2.height;
                        }
                        if (this.isGlue(component2)) {
                            preferredSize2.width += n5;
                        }
                        component2.setBounds(leftToRight ? n4 : (n4 - preferredSize2.width), top, preferredSize2.width, height);
                        n4 = (leftToRight ? (n4 + preferredSize2.width) : (n4 - preferredSize2.width));
                    }
                }
            }
            else {
                int y = toolBar.isFloatable() ? SynthIcon.getIconHeight(SynthToolBarUI.this.handleIcon, context) : 0;
                SynthToolBarUI.this.contentRect.x = 0;
                SynthToolBarUI.this.contentRect.y = y;
                SynthToolBarUI.this.contentRect.width = toolBar.getWidth();
                SynthToolBarUI.this.contentRect.height = toolBar.getHeight() - y;
                int left = insets.left;
                int width = toolBar.getWidth() - insets.left - insets.right;
                int y2 = y + insets.top;
                int n2 = 0;
                if (n > 0) {
                    n2 = (toolBar.getHeight() - this.minimumLayoutSize(container).height) / n;
                    if (n2 < 0) {
                        n2 = 0;
                    }
                }
                for (int j = 0; j < toolBar.getComponentCount(); ++j) {
                    Component component = toolBar.getComponent(j);
                    if (component.isVisible()) {
                        Dimension preferredSize = component.getPreferredSize();
                        if (preferredSize.width >= width || component instanceof JSeparator) {
                        }
                        else {
                            left = left + width / 2 - preferredSize.width / 2;
                            width = preferredSize.width;
                        }
                        if (this.isGlue(component)) {
                            preferredSize.height += n2;
                        }
                        component.setBounds(left, y2, width, preferredSize.height);
                        y2 += preferredSize.height;
                    }
                }
            }
            context.dispose();
        }
        
        private boolean isGlue(Component component) {
            if (component.isVisible() && component instanceof Box.Filler) {
                Box.Filler filler = (Box.Filler)component;
                Dimension minimumSize = filler.getMinimumSize();
                Dimension preferredSize = filler.getPreferredSize();
                return minimumSize.width == 0 && minimumSize.height == 0 && preferredSize.width == 0 && preferredSize.height == 0;
            }
            return false;
        }
    }
}
